@@ -3,19 +3,33 @@
 import pwd
 import re
 import locale
+import sys
 from os import makedirs, unlink, stat
 from os.path import join, isfile, isdir, ismount, exists, realpath
-from configobj import ConfigObj
 from pickle import dump, load
 from time import sleep, time
 from datetime import datetime, timedelta
+from configobj import ConfigObj
+
+if sys.version_info[0] >= 3:
+    unicode = str
+    open_mode = 'wb'
+    read_mode = 'rb'
+else:
+    open_mode = 'w'
+    read_mode = 'r'
+
+try:
+    import ConfigParser
+except:
+    from configparser import ConfigParser
 
-import ConfigParser
-from pyeole.service import manage_service
-from pyeole.process import system_out, tcpcheck
 from creole.client import CreoleClient, CreoleClientError
 from creole.loader import creole_loader, config_save_values
 from creole.template import CreoleTemplateEngine
+from pyeole.service import manage_services
+from pyeole.process import system_out, tcpcheck
+from pyeole.ansiprint import print_title, print_orange
 
 BAREOS_CONF = "/etc/eole/bareos.conf"
 BCONSOLE_CONF = '/etc/bareos/bconsole.conf'
@@ -91,10 +105,10 @@ def reload_conf(test='both'):
         ret, stdout, stderr = system_out(cmd)
         if ret != 0:
             raise Exception(u'ERREUR : erreur au test de la configuration : {0}, {1}'.format(stdout, stderr))
-        manage_service('restart', 'bareos-storage')
+        manage_services('restart', 'bareos-storage')
 
 def _save_bareos_conf(filename, dic):
-    fh = file(filename, 'w')
+    fh = open(filename, 'w')
     cfgparser = ConfigParser.ConfigParser()
     for section, val in dic.items():
         cfgparser.add_section(section)
@@ -196,7 +210,7 @@ def test_mount_point(dic):
         mount_point_busy = True
 
     # test d'écriture si le montage est le bon (drapeau right_rights)
-    if right_node_mounted == True: # implique mount_point_busy == True
+    if right_node_mounted is True: # implique mount_point_busy == True
         try:
             system_out_catched(['su', '-', 'bareos', '-c', "touch {0}".format(TEST_MOUNT_FILE), '-s', '/bin/bash'])
             system_out_catched(['su', '-', 'bareos', '-c', "rm -f {0}".format(TEST_MOUNT_FILE), '-s', '/bin/bash'])
@@ -285,7 +299,7 @@ def get_mount_command(dic, uid):
                 raise Exception(u"La machine {0} n'est pas accessible sur les ports 139 et 445".format(dic['smb_ip']))
         DISTANT_LOGIN_MOUNT = bareos_config.get('DISTANT_LOGIN_MOUNT', '/bin/mount -t cifs -o username={0},password={1},ip={2},uid={3},noexec,nosuid,nodev,vers=3.0 //{4}/{5} {6}')
         DISTANT_MOUNT = bareos_config.get('DISTANT_MOUNT', '/bin/mount -t cifs -o password={0},ip={1},uid={2},noexec,nosuid,nodev,vers=3.0 //{3}/{4} {5}')
-        if dic['smb_login'] != None:
+        if dic['smb_login'] is not None:
             cmd = DISTANT_LOGIN_MOUNT.format(dic['smb_login'],
                     dic['smb_password'], dic['smb_ip'], uid,
                     dic['smb_machine'], dic['smb_partage'], MOUNT_POINT).split()
@@ -414,7 +428,7 @@ def run_bareos_cmd(stdin):
     return system_out(cmd, stdin=stdin)
 
 
-def bareos_query(query, *args):
+def bareos_query(query, *args, empty_not_allowed=False):
     """
     Return bareos query command result
     :param query: query id as specified in /etc/bareos/query.sql
@@ -435,24 +449,24 @@ def bareos_query(query, *args):
         resu = bareos_query_re.search(res)
         if resu:
             res = res.split(resu.group())
-        else:
-            return  []
-        header = [title.strip() for title in res[1].split('|') if title != '\n']
-        results = [[j.strip() for j in i.split('|') if j != ''] for i in res[2].split('\n') if i != '']
-        for result in results:
-            result_dict = {}
-            for value, key in zip(result, header):
-                result_dict[key] = value
-            results_list.append(result_dict)
+            header = [title.strip() for title in res[1].split('|') if title != '\n']
+            results = [[j.strip() for j in i.split('|') if j != ''] for i in res[2].split('\n') if i != '']
+            for result in results:
+                result_dict = {}
+                for value, key in zip(result, header):
+                    result_dict[key] = value
+                results_list.append(result_dict)
     except AttributeError:
         raise Exception('Résultat de la commande bconsole imprévu')
+    if empty_not_allowed and not results_list:
+        raise Exception('Aucun résultat\nstatus : {}\nstdout : {}\nstderr : {}'.format(status, res, err))
     return results_list
 
 
 def _empty_file(filename):
     if isfile(filename):
         unlink(filename)
-    fh = file(filename,'w')
+    fh = open(filename,'w')
     fh.write('')
     fh.close()
 
@@ -470,7 +484,7 @@ def run_backup(level='Full', when='', bareos_fd=None):
        load_bareos_support()['support_type'] == 'none':
         raise Exception("Le support de sauvegarde local n'est pas configuré")
     if when != '':
-        if re.match(r'[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}', when) == None:
+        if re.match(r'[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}', when) is None:
             raise Exception('Horaire mal renseigné')
         else:
             when = ' When=\"{0}\"'.format(when)
@@ -499,7 +513,7 @@ def load_bareos_support():
 def save_bareos_support_usb(usb_path, no_reload=False):
     if not bareos_active_sd():
         raise Disabled(u'Bareos sd désactivé')
-    if usb_path == None:
+    if usb_path is None:
         raise Exception(u'Chemin usb inconnu')
     config = creole_loader(load_extra=True, rw=True, owner='bareos')
     config.bareos.support.support_type = u'usb'
@@ -514,16 +528,16 @@ def save_bareos_support_smb(smb_machine, smb_ip,
     if not bareos_active_sd():
         raise Disabled(u'Bareos sd désactivé')
 
-    if smb_machine == None or smb_ip  == None or smb_partage == None:
+    if smb_machine is None or smb_ip is None or smb_partage is None:
         raise Exception(u'Nom machine, IP ou nom du partage inconnu')
 
     config = creole_loader(load_extra=True, rw=True, owner='bareos')
     config.bareos.support.support_type = u'smb'
     smb_login = unicode_or_none(smb_login)
     smb_password = unicode_or_none(smb_password)
-    if smb_login != None and smb_password == None:
+    if smb_login is not None and smb_password is None:
         raise Exception(u'Login spécifié mais password absent')
-    if smb_login == None and smb_password != None:
+    if smb_login is None and smb_password is not None:
         raise Exception(u'Password spécifié mais login absent')
 
     config.bareos.support.smb_machine = unicode(smb_machine)
@@ -566,56 +580,78 @@ def test_space_left():
         raise Exception(u"Impossible de vérifier l'espace disponible dans " + MOUNT_POINT)
     line = stdout.split()
 
-    mnt_free_space = int(line[-1]) * 1024
-    print(u"Espace disponible sur le support : {} ({})".format(sizeof_fmt(mnt_free_space), mnt_free_space))
+    print_title('Espace disponible')
 
+    mnt_free_space = int(line[-1]) * 1024
     vols = bareos_query('space_left')
     recycle_free_space = 0
     for vol in vols:
-        if vol['VolStatus'] == 'Append':
+        if vol['volstatus'] == 'Append':
             pass
         else:
-            recycle_free_space += int(vol['MaxVolBytes'].replace(',',''))
-    print(u"Espace disponible pouvant être réutilisé : {} ({})".format(sizeof_fmt(recycle_free_space), recycle_free_space))
+            recycle_free_space += int(vol['maxvolbytes'].replace(',',''))
 
     total_free_space = mnt_free_space + recycle_free_space
-    print(u"Espace disponible total : {} ({})".format(sizeof_fmt(total_free_space), total_free_space))
+    available_space = [
+    "Espace disponible sur le support : {} ({})".format(sizeof_fmt(mnt_free_space), mnt_free_space),
+    "Espace disponible pouvant être réutilisé : {} ({})".format(sizeof_fmt(recycle_free_space), recycle_free_space),
+    "Espace disponible total : {} ({})".format(sizeof_fmt(total_free_space), total_free_space)
+    ]
 
-    estimate_output = '0'
-    estimate_output_date = ""
+    for s in available_space:
+        print(s)
+
+
+    print_title("Dernière estimation de la sauvegarde Totale :")
+    estimate_output_date = None
+    full_size_estimate_msg = None
     if isfile(BAREOS_RAPPORT_FSL):
         with open(BAREOS_RAPPORT_FSL) as f:
             data = f.read()
-        estimate_output, estimate_output_date = data.split(" : ")
+        if ' : ' in data:
+            estimate_output, estimate_output_date = data.split(' : ')
+            if estimate_output and estimate_output != "None":
+                full_size_estimate = int(estimate_output.replace(',',''))
+                full_size_estimate_msg = "{} ({}) Calculé le {}".format(sizeof_fmt(full_size_estimate), full_size_estimate, estimate_output_date)
 
-    if estimate_output and estimate_output != "None":
-        full_size_estimate = int(estimate_output.replace(',',''))
+    if full_size_estimate_msg is None:
+        print_orange("Aucune estimation disponible : lancer le script /usr/share/eole/sbin/bareosfreespaceleft.py pour la calculer.")
+        full_size_estimate = 0
     else:
-        try:
-            full_size_estimate = int(bareos_query('last_full_backups')[0]['JobBytes'].replace(',',''))
-        except:
-            full_size_estimate = 0
+        print(full_size_estimate_msg)
+
+
+    print_title("Tailles des dernières sauvegardes :")
+
+    last_full_size = bareos_query('last_full_backups')
+    try:
+        last_full_size = int(last_full_size[0]['result'].replace(',',''))
+    except:
+        last_full_size = 0
+    last_full_size_msg = "Totale (dernière) : {} ({})".format(sizeof_fmt(last_full_size), last_full_size)
 
     avg_last_diff = bareos_query('avg_last_diff_backups')
     try:
-        avg_last_diff = float(avg_last_diff[0]['AVG(JobBytes)'].replace(',',''))
+        avg_last_diff = float(avg_last_diff[0]['result'].replace(',',''))
     except:
         avg_last_diff = 0
+    avg_last_diff_msg = "Différentielle (moyenne) : {} ({})".format(sizeof_fmt(avg_last_diff), avg_last_diff)
+
     avg_last_incr = bareos_query('avg_last_incr_backups')
     try:
-        avg_last_incr = float(avg_last_incr[0]['AVG(JobBytes)'].replace(',',''))
+        avg_last_incr = float(avg_last_incr[0]['result'].replace(',',''))
     except:
         avg_last_incr = 0
+    avg_last_incr_msg = "Incrémentale (moyenne): {} ({})".format(sizeof_fmt(avg_last_incr), avg_last_incr)
 
-    print(u"Tailles des dernières sauvegardes :")
 
-    print(u"Full : {} ({}) Calculé le {}".format(sizeof_fmt(full_size_estimate), full_size_estimate, estimate_output_date))
-    print(u"Différentielle (moyenne) : {} ({})".format(sizeof_fmt(avg_last_diff), avg_last_diff))
-    print(u"Incrémentale (moyenne): {} ({})".format(sizeof_fmt(avg_last_incr), avg_last_incr))
+    print(last_full_size_msg)
+    print(avg_last_diff_msg)
+    print(avg_last_incr_msg)
 
-    dic = load(file(BAREOS_RAPPORT, 'r'))
+    dic = load(open(BAREOS_RAPPORT, read_mode))
     try:
-        dic = load(file(BAREOS_RAPPORT, 'r'))
+        dic = load(open(BAREOS_RAPPORT, read_mode))
     except (EOFError, IOError):
         dic = {}
 
@@ -623,7 +659,7 @@ def test_space_left():
     dic['backup_size'] = {'last_full': full_size_estimate, 'last_fullH': sizeof_fmt(full_size_estimate),
                          'avg_last_diff': avg_last_diff, 'avg_last_diffH': sizeof_fmt(avg_last_diff),
                          'avg_last_incr': avg_last_incr, 'avg_last_incrH': sizeof_fmt(avg_last_incr)}
-    dump(dic, file(BAREOS_RAPPORT, 'w'))
+    dump(dic, open(BAREOS_RAPPORT, open_mode), protocol=2)
 
 
 def sizeof_fmt(num, suffix='B'):
@@ -641,7 +677,7 @@ def test_bareos_support(force=False, chown=False):
 
     if force or bareos_active_sd():
         status, dic = mount_bareos_support(retry=2, chown=chown)
-        if status == True:
+        if status is True:
             umount_bareos_support()
         comment = mount_status_to_str(dic)
         test_tmp_pcent()
@@ -697,7 +733,7 @@ def load_bareos_jobs(): #check_active=True):
 #def save_bareos_job(dic):
 #    if not bareos_active_dir():
 #        raise Disabled(u'Bareos dir désactivé')
-#    dump(dic, file(BAREOS_JOB, 'w'))
+#    dump(dic, open(BAREOS_JOB, 'w'))
 
 def write_bareos_schedule():
     """
@@ -710,11 +746,11 @@ def write_bareos_schedule():
         except CreoleClientError:
             pass
     if not isfile(INCLUDE_CONF):
-        fh = file(INCLUDE_CONF, 'w+')
+        fh = open(INCLUDE_CONF, 'w+')
         fh.write('\n')
         fh.close()
     if not isfile(RESTORE_CONF):
-        fh = file(RESTORE_CONF, 'w+')
+        fh = open(RESTORE_CONF, 'w+')
         fh.write('\n')
         fh.close()
     try:
@@ -879,7 +915,7 @@ def bareos_get_jobs_list():
 
 def pprint_bareos_jobs():
     jobs = display_bareos_jobs()
-    if jobs != None:
+    if jobs is not None:
         pprinted_bareos_jobs = "\n".join(["\t{0}".format(job) for num, job in display_bareos_jobs()])
     else:
         pprinted_bareos_jobs = "\tAucun job programmé."
@@ -915,22 +951,22 @@ def _is_more_than_one_day(epoch):
 def _bareos_rapport(job_type, text, status):
     date = time()
     try:
-        dic = load(file(BAREOS_RAPPORT, 'r'))
+        dic = load(open(BAREOS_RAPPORT, read_mode))
     except:
         dic = {}
     dic[job_type] = {'text' : text,
         'date': date,
         'status': str(status)}
-    dump(dic, file(BAREOS_RAPPORT, 'w'))
+    dump(dic, open(BAREOS_RAPPORT, open_mode), protocol=2)
 
 def _bareos_pcent_usage():
     try:
-        dic = load(file(BAREOS_RAPPORT, 'r'))
+        dic = load(open(BAREOS_RAPPORT, read_mode))
     except:
         dic = {}
     cmd = ['df', MOUNT_POINT, '--output=pcent,ipcent']
     ret, stdout, stderr = system_out(cmd)
-    if ret is not 0:
+    if ret != 0:
         pcent, ipcent = -1, -1
     else:
         value = stdout.split()
@@ -944,7 +980,7 @@ def _bareos_pcent_usage():
             except:
                 pcent, ipcent = -1, -1
     dic['disk'] = {'pcent': pcent, 'ipcent': ipcent}
-    dump(dic, file(BAREOS_RAPPORT, 'w'))
+    dump(dic, open(BAREOS_RAPPORT, open_mode), protocol=2)
 
 
 
@@ -960,32 +996,31 @@ def bareos_rapport_err(job_type):
 def bareos_rapport_load(report='sauvegarde'):
     if not isfile(BAREOS_RAPPORT) or stat(BAREOS_RAPPORT).st_size == 0:
         return (BAREOS_RAPPORT_UNKNOWN, 'Aucune sauvegarde')
-    else:
-        try:
-            info = load(file(BAREOS_RAPPORT, 'r'))[report]
-        except KeyError as e:
-            return (BAREOS_RAPPORT_UNKNOWN, 'Aucune sauvegarde')
-        except:
-            return (BAREOS_RAPPORT_UNKNOWN, "Impossible de lire le rapport de sauvegarde")
+    try:
+        info = load(open(BAREOS_RAPPORT, read_mode))[report]
+    except KeyError as e:
+        return (BAREOS_RAPPORT_UNKNOWN, 'Aucune sauvegarde')
+    except:
+        return (BAREOS_RAPPORT_UNKNOWN, "Impossible de lire le rapport de sauvegarde")
 
-        status = info['status']
-        text = info['text'].format(_date_from_epoch(info['date']))
-        if status == BAREOS_RAPPORT_UNKNOWN:
-            #verifie si l'etat de sauvegarde n'a pas plus de 24 heures
-            if _is_more_than_one_day(info['date']):
-                status = BAREOS_RAPPORT_ERR
-                text = "Sauvegarde démarrée depuis plus d'une journée (date de démarrage : %s)"% _date_from_epoch(info['date'])
-        elif status not in [BAREOS_RAPPORT_UNKNOWN, BAREOS_RAPPORT_OK, BAREOS_RAPPORT_ERR]:
-            status = BAREOS_RAPPORT_UNKNOWN
-            text = 'Status inconnu : %s' % info['status']
-        return (status, text)
+    status = info['status']
+    text = info['text'].format(_date_from_epoch(info['date']))
+    if status == BAREOS_RAPPORT_UNKNOWN:
+        #verifie si l'etat de sauvegarde n'a pas plus de 24 heures
+        if _is_more_than_one_day(info['date']):
+            status = BAREOS_RAPPORT_ERR
+            text = "Sauvegarde démarrée depuis plus d'une journée (date de démarrage : %s)"% _date_from_epoch(info['date'])
+    elif status not in [BAREOS_RAPPORT_UNKNOWN, BAREOS_RAPPORT_OK, BAREOS_RAPPORT_ERR]:
+        status = BAREOS_RAPPORT_UNKNOWN
+        text = 'Status inconnu : %s' % info['status']
+    return (status, text)
 
 
 def bareos_rapport_load_pcent_usage():
     if not isfile(BAREOS_RAPPORT) or stat(BAREOS_RAPPORT).st_size == 0:
         return (-1, -1)
     try:
-        info = load(file(BAREOS_RAPPORT, 'r'))['disk']
+        info = load(open(BAREOS_RAPPORT, read_mode))['disk']
     except:
         return (-1, -1)
 
@@ -998,8 +1033,8 @@ def bareos_rapport_load_free_space():
     if not isfile(BAREOS_RAPPORT) or stat(BAREOS_RAPPORT).st_size == 0:
         return ("-1", "pas suffisamment d'éléments pour faire une estimation")
     try:
-        free_space = load(file(BAREOS_RAPPORT, 'r'))['free_space']
-        backup_size = load(file(BAREOS_RAPPORT, 'r'))['backup_size']
+        free_space = load(open(BAREOS_RAPPORT, read_mode))['free_space']
+        backup_size = load(open(BAREOS_RAPPORT, read_mode))['backup_size']
     except:
         return ("-1", "pas suffisamment d'éléments pour faire une estimation")
 
@@ -1013,4 +1048,3 @@ def bareos_rapport_load_free_space():
         status = "1"
         text = 'Espace disponible ({}) suffisant pour la sauvegarde'.format(free_space['total_free_spaceH'])
     return (status, text)
-
