@@ -47,6 +47,7 @@ import logging
 import shutil
 
 from subprocess import Popen, PIPE
+from packaging.version import parse as version_parse
 
 try:
     # python3
@@ -172,7 +173,10 @@ class IsoEOLE(object):
         text = msg.format(release=self.release)
         self.logger.debug(text)
 
-        iso_name_format = 'eole-{point_release}-alternate-amd64.iso'
+        if version_parse(self.release) < version_parse('2.9.0'):
+            iso_name_format = 'eole-{point_release}-alternate-amd64.iso'
+        else:
+            iso_name_format = 'eole-{point_release}-amd64.iso'
 
         self._cache['iso_name'] = iso_name_format.format(point_release=self.last_point_release)
         return self._cache['iso_name']
