@@ -22,6 +22,7 @@ A :class:`trace` decorator is provided and can be enabled by setting
 
 """
 
+import sys
 import logging
 import logging.config
 
@@ -32,6 +33,11 @@ from pyeole.decorator import advice, deprecated
 
 from pyeole.inspect_utils import get_caller_infos, format_caller
 
+
+if sys.version_info[0] >= 3:
+    unicode = str
+
+
 ENABLE_TRACE = False
 """Enable tracing globally."""
 
@@ -347,7 +353,7 @@ def set_formatter(logger, handler, formatter):
         >>> from pyeole.log import init_logging, set_formatter
         >>> log = init_logging(level=u'info')
         >>> set_formatter(log, u'stderr', u'brief')
-        >>> log.warn(u"ATTENTION: be carefull")
+        >>> log.warning(u"ATTENTION: be carefull")
 
     :param logger: logger object
     :type logger: `logging.Logger`
@@ -389,7 +395,7 @@ def set_filters(logger, handler, filters):
         >>> log = init_logging(level=u'debug', console=[u'stdout', u'stderr'],
                                filename=u'/tmp/debug.log')
         >>> set_filters(log, u'file', [u'debug'])
-        >>> log.warn(u"ATTENTION: this message is display on stderr")
+        >>> log.warning(u"ATTENTION: this message is display on stderr")
         >>> log.debug(u"ATTENTION: this message is output in /tmp/debug.log")
 
     :param logger: logger object
@@ -532,37 +538,31 @@ EOLE_FORMAT = "%(asctime)s - %(name)s - %(levelname)s - '%(funcName)s' %(message
 DEFAULT_LOG_FILE = "/var/log/eole/default.log"
 DEFAULT_LOG_LEVEL = 'ERROR'
 
-@deprecated("use pyeole.log.init_logging")
 class DebugFilter(logging.Filter):
     def filter(self, rec):
         if rec.levelno <= logging.DEBUG:
             return logging.Filter.filter(self, rec)
 
-@deprecated("use pyeole.log.init_logging")
 class InfoFilter(logging.Filter):
     def filter(self, rec):
         if logging.INFO >= rec.levelno > logging.DEBUG:
             return logging.Filter.filter(self, rec)
 
-@deprecated("use pyeole.log.init_logging")
 class DebugInfoFilter(logging.Filter):
     def filter(self, rec):
         if logging.INFO >= rec.levelno:
             return logging.Filter.filter(self, rec)
 
-@deprecated("use pyeole.log.init_logging")
 class ErrorFilter(logging.Filter):
     def filter(self, rec):
         if rec.levelno >= logging.WARNING:
             return logging.Filter.filter(self, rec)
 
-@deprecated("use pyeole.log.init_logging")
 def get_formatter(logfmt, datefmt):
     if datefmt:
         return logging.Formatter(logfmt, datefmt)
     return logging.Formatter(logfmt)
 
-@deprecated("use pyeole.log.init_logging")
 def get_log_level(level=DEFAULT_LOG_LEVEL):
     """
         permet de renvoyer un objet loglevel avec un comportement standard
@@ -576,7 +576,6 @@ def get_log_level(level=DEFAULT_LOG_LEVEL):
         return getattr(logging, level)
     return logging.INFO
 
-@deprecated("use pyeole.log.init_logging")
 def make_file_handler(logfile, loglevel, logfilter, logformat, datefmt):
     """
 
@@ -595,7 +594,6 @@ def make_file_handler(logfile, loglevel, logfilter, logformat, datefmt):
         handler.addFilter(logfilter)
     return handler
 
-@deprecated("use pyeole.log.init_logging")
 def make_logger(packagename, logfile=DEFAULT_LOG_FILE, errfile=None, debugfile=None, loglevel=DEFAULT_LOG_LEVEL, splitlog=False, logfmt=EOLE_FORMAT, datefmt=None):
     """
 
