@@ -26,7 +26,10 @@ import argparse
 import pty
 import json
 import logging
-import urlparse
+try:
+    import urlparse
+except Exception:
+    import urllib.parse as urlparse
 import tempfile
 from shutil import rmtree
 from time import time
@@ -57,7 +60,6 @@ if not CreoleClient.is_in_lxc():
     from creole.containers import is_lxc_running
     from creole.error import VirtError
     from creole.template import CreoleTemplateEngine
-    from creole.eosfunc import list_cdrom_devices
 
 log = logging.getLogger(__name__)
 log.addHandler(logging.NullHandler())
@@ -86,6 +88,8 @@ _SOURCES_LISTS_SAMBA = os.path.join(_SOURCES_LISTS_DIR, 'seth-samba.list')
 _APT_CONFIG_DIR = u'/etc/apt/apt.conf.d/'
 _APT_PROXY_FILE = os.path.join(_APT_CONFIG_DIR, '02eoleproxy')
 
+_EOLEHUB_COMMON_ENV = u'/etc/eole/containers.conf.d/common.env'
+
 _APT_TERM_FILE = u'/var/log/apt/term.log'
 """Default log file for apt-get standard output
 """
@@ -114,12 +118,6 @@ _SOURCES_CONF = {u'ubuntu': {u'mirror': u'http://eole.ac-dijon.fr/ubuntu',
                            u'level': {u'proposed-updates': [u'stable', u'security', u'updates'],
                                       u'updates': [u'stable', u'security'],
                                       u'security': [u'stable']}},
-                 u'cdrom': {u'mirror': u'file:/cdrom/',
-                            u'sources_file':  u'/etc/apt/sources.list',
-                            u'level': {u'proposed-updates': None,
-                                       u'updates': None,
-                                       u'security': None,
-                                       u'stable': None}}
                 }
 _MIRROR_DIST = {'Ubuntu': '{0}/main/binary-amd64'.format(UBUNTU_VERSION),
                 'EOLE': 'eole-{0}-unstable'.format(EOLE_VERSION),
@@ -128,8 +126,6 @@ _MIRROR_DIST = {'Ubuntu': '{0}/main/binary-amd64'.format(UBUNTU_VERSION),
 _ARCHIVE_DIR = '/var/cache/apt/archives/'
 _LISTS_DIR = '/var/lib/apt/lists'
 
-_MOUNT_POINT = '/media/cdrom'
-
 #valeurs par défaut si impossible de récupérer celles configurées
 _EOLE_MIRRORS = ['eole.ac-dijon.fr', 'ftp.crihan.fr']
 _ENVOLE_MIRRORS = ['eole.ac-dijon.fr', 'ftp.crihan.fr']
@@ -143,7 +139,7 @@ DEFAULT_LEVEL = u'updates'
 CREOLE_ERRORS = (NotFoundError, TimeoutCreoleClientError, CreoleClientError)
 
 
-class PackageError(StandardError):
+class PackageError(Exception):
     """Base exception for package management
 
     """
@@ -259,26 +255,6 @@ def parse_apt_cmdline():
     eole.set_defaults(sources_file=u'/etc/apt/sources.list.d/eole.list')
     eole.set_defaults(func=u'configure')
     eole.set_defaults(func_args=configure_args)
-    cdrom = vendor.add_parser(u'cdrom', help=_(u'CDROM sources.list'))
-    cdrom.add_argument(u'-m', u'--mirror', metavar=u'URI', type=str,
-                       default=u'file:/media/cdrom',
-                       help=_(u'URI of the mirror'))
-
-    cdrom.add_argument(u'-d', u'--dist', type=str,
-                       choices=[UBUNTU_VERSION],
-                       default=UBUNTU_VERSION, help=_(u'distribution name'))
-
-    cdrom.add_argument(u'-c', u'--component', nargs=u'+',
-                       choices=[u'universe', u'main'],
-                       default=u'universe',
-                       help=_(u'component to activate'))
-    cdrom.add_argument(u'-l', u'--level',
-                       choices=(u'stable'),
-                       default=u'updates',
-                       help=_(u'update level'))
-    cdrom.set_defaults(func=u'configure')
-    cdrom.set_defaults(func_args=configure_args)
-
     update = actions.add_parser(u'update', help=_(u'update list of indexes'))
     update.set_defaults(func=u'update')
 
@@ -294,6 +270,12 @@ def parse_apt_cmdline():
     list_upgrade = actions.add_parser(u'list-upgrade', help=_(u'list packages to upgrade'))
     list_upgrade.set_defaults(func=u'list_upgrade')
 
+    fetch_archives = actions.add_parser(u'fetch_archives', help=_(u'download package'))
+    fetch_archives.add_argument(u'packages', metavar=u'packages', type=str, nargs=u'+',
+                                help=_(u'package names'))
+    fetch_archives.set_defaults(func=u'fetch_archives')
+    fetch_archives.set_defaults(func_args=pkg_args)
+
     install = actions.add_parser(u'install', help=_(u'install packages'))
     install.add_argument(u'packages', metavar=u'package', type=str, nargs=u'+',
                          help=_(u'package names'))
@@ -399,6 +381,8 @@ def _build_apt_cmd_line(**kwargs):
         cmd.extend([u'eole-lxc-container-pkg'] + kwargs[u'packages'])
     elif action == u'remove':
         cmd.extend(kwargs[u'packages'])
+    elif action == u'fetch_archives':
+        cmd.extend(kwargs[u'packages'])
 
     return cmd
 
@@ -427,15 +411,19 @@ def _run_apt_get(arg_list, silent=False, display_info=None):
         if code != 0:
             stderr = proc.stderr.read()
     if code != 0:
+        if isinstance(stderr, bytes):
+            stderr = stderr.decode('UTF-8')
         # ticket #17482 make error message great again
-        if stderr.strip() == "E: There were unauthenticated packages and -y was used without --allow-unauthenticated":
-            stderr = _(u"There were unauthenticated packages and Maj-Auto has prevented their installation").encode('UTF-8')
+        unauthenticated_msg = ["E: There were unauthenticated packages and -y was used without --allow-unauthenticated",
+                "E: Il y a eu des paquets non authentifiés et -y a été employé sans --allow-unauthenticated"]
+        if stderr.strip() in unauthenticated_msg:
+            stderr = _(u"There were unauthenticated packages and Maj-Auto has prevented their installation")
         elif "Connexion à localhost: 3142 (127.0.0.1) impossible." in stderr.strip():
-            stderr = stderr+"\n"+_APT_CACHER_ERROR_MSG.encode('UTF-8')
+            stderr = stderr+"\n"+_APT_CACHER_ERROR_MSG
         elif "Impossible de se connecter à 192.0.2.1:3142" in stderr.strip():
-            stderr = stderr+"\n"+_APT_CACHER_ERROR_MSG_CONTAINER.encode('UTF-8')
+            stderr = stderr+"\n"+_APT_CACHER_ERROR_MSG_CONTAINER
         raise AptError(_(u"apt-get ({0}) returned an error (code {1}). See {2}\n{3}")\
-                .format(u" ".join(cmd[1:]), code, _APT_TERM_FILE, stderr.decode('UTF-8')))
+                .format(u" ".join(cmd[1:]), code, _APT_TERM_FILE, stderr))
     return code
 
 def apt_source_get_uri(line):
@@ -596,7 +584,6 @@ class EolePkgApt(object):
         # Do not fail on containers if not instanciated
         self._instantiated = None
 
-        self._cdrom_devices = None
         self._apt_cacher_port = None
 
         # Do not run in container mode within a container
@@ -610,27 +597,18 @@ class EolePkgApt(object):
         self.set_test_run(_TESTRUN)
         self.set_test_run(test)
         self.groups = None
-        self._cdrom_mounted = False
         self.ignore = ignore
 
-    def is_cdrom_devices(self):
-        if self._cdrom_devices is None:
-            try:
-                self._cdrom_devices = self.client.get_creole('cdrom_devices')
-            except CREOLE_ERRORS as err:
-                if self.ignore:
-                    self._cdrom_devices = list_cdrom_devices()
-                else:
-                    raise err
-        return self._cdrom_devices
-
     def is_container_mode(self):
         if self._container_mode is None:
             try:
                 self._container_mode = not self.client.is_in_lxc()\
                                       and self.client.get_creole(u'mode_conteneur_actif') == u'oui'
+                if self.ignore:
+                    self.get_groups()
             except CREOLE_ERRORS as err:
                 if self.ignore:
+                    self._container_mode = None
                     return False
                 else:
                     raise err
@@ -720,7 +698,7 @@ class EolePkgApt(object):
         if diff[0] != 0:
             # FIXME: warn or info ?
             c_name = ctx['name']
-            log.warn(_(u'Container {0} has specific sources.list').format(c_name))
+            log.warning(_(u'Container {0} has specific sources.list').format(c_name))
             cmd = ['apt-eole', 'update']
             log.debug(u'Run: {0} ({1})'.format(' '.join(cmd), c_name))
             code, stdout, stderr = system_progress_out(cmd, container=ctx,
@@ -765,7 +743,7 @@ class EolePkgApt(object):
                             if self.is_instantiated():
                                 raise VirtError(msg)
                             else:
-                                log.warn(msg)
+                                log.warning(msg)
                                 continue
 
                         outputs[container] = getattr(self, fast_func)(silent=silent, ctx=container_infos, **kwargs)
@@ -803,7 +781,7 @@ class EolePkgApt(object):
                             if self.is_instantiated():
                                 raise VirtError(msg)
                             else:
-                                log.warn(msg)
+                                log.warning(msg)
                                 continue
                         msg = _(u'Running {0} for container {1}').format(action, container)
                         log.debug(u'Run: {0} ({1})'.format(' '.join(cmd), container))
@@ -973,49 +951,8 @@ class EolePkgApt(object):
             raise RepositoryError(msg.format(release_url))
 
 
-    def _mount_cdrom(self):
-        """Mount installer CDROM
-
-        Use first CDROM with a “dists/” top level directory.
-        """
-        if not os.path.isdir(_MOUNT_POINT):
-            os.makedirs(_MOUNT_POINT)
-        if not os.path.ismount(_MOUNT_POINT):
-            for cdrom in self.is_cdrom_devices():
-                cmd = ['/bin/mount', cdrom, _MOUNT_POINT, '-o', 'ro']
-                if system_code(cmd) != 0:
-                    continue
-                elif not os.path.isdir('{0}/dists'.format(_MOUNT_POINT)):
-                    self._umount_cdrom(force=True)
-
-            if os.path.ismount(_MOUNT_POINT):
-                log.debug("Run: {0}".format(" ".join(cmd)))
-                self._cdrom_mounted = True
-            else:
-                raise SystemError(_('No installer CDROM found'))
-        else:
-            log.debug("{0} already mount".format(_MOUNT_POINT))
-
-    def _umount_cdrom(self, force=False):
-        if (force or self._cdrom_mounted) and os.path.ismount(_MOUNT_POINT):
-            cmd = ['/bin/umount', _MOUNT_POINT]
-            log.debug("Run: {0}".format(" ".join(cmd)))
-            if system_code(cmd) != 0:
-                raise SystemError(_('unable to umount {0}').format(_MOUNT_POINT))
-
-
     def _prepare_cache(self):
         self._load_apt_cache()
-        for cache in self.cache._list.list:
-            if cache.uri.startswith('cdrom:['):
-                log.info(_('Update cdrom repository'))
-                _configure_sources_mirror(self, cdrom=True)
-                self._prepare_cache()
-                break
-            if cache.uri.startswith('file:'+_MOUNT_POINT):
-                self._mount_cdrom()
-                return
-
 
     def get_upgradable_list(self, silent=True):
         """Search all packages who needs to be updated.
@@ -1076,7 +1013,6 @@ class EolePkgApt(object):
 
         self._prepare_cache()
         code = _run_apt_get(args + apply_to, silent=silent)
-        self._umount_cdrom()
         return code
 
 
@@ -1124,7 +1060,6 @@ class EolePkgApt(object):
 
         self._prepare_cache()
         ret_code = _run_apt_get([u'update'], silent=silent)
-        self._umount_cdrom()
         if self.cache is not None:
             self.cache.open()
         return ret_code
@@ -1140,7 +1075,7 @@ class EolePkgApt(object):
         ret = system_out(['apt-get', 'install', '-f'])
         if ret[0] != 0:
             raise AptError (_(u"apt-get install -f returned an error\n{0}")\
-                    .format(''.join(ret[1:]).decode('UTF-8')))
+                    .format(''.join(ret[1:])))
         return 0
 
 
@@ -1164,7 +1099,6 @@ class EolePkgApt(object):
 
         self._prepare_cache()
         code = _run_apt_get(action, silent=silent)
-        self._umount_cdrom()
 
         if code == 0 and not download_only and not simulate:
             self.clean()
@@ -1188,7 +1122,6 @@ class EolePkgApt(object):
         """
         self._prepare_cache()
         code = _run_apt_get([u'-d', u'install'] + packages, silent=silent)
-        self._umount_cdrom()
         return code
 
 
@@ -1229,7 +1162,6 @@ class EolePkgApt(object):
         os.chdir(dest)
         self._prepare_cache()
         ret = _run_apt_get([u'download', pkgname], silent=silent)
-        self._umount_cdrom()
         os.chdir(old_dir)
         return ret
 
@@ -1242,7 +1174,6 @@ class EolePkgApt(object):
         """
         self._prepare_cache()
         code = _run_apt_get([u'clean'])
-        self._umount_cdrom()
         return code
 
 
@@ -1654,9 +1585,9 @@ class EolePkg(object):
 
 # FIXME: Remove ignore argument since it's extracted from pkgmgr
 def _configure_sources_mirror(pkgmgr, ubuntu=None, eole=None, envole=None,
-                              level='stable', ignore=False, cdrom=False,
+                              level='stable', ignore=False,
                               release=None, envole_release=None,
-                              eole_level=None, envole_level=None):
+                              eole_level=None, envole_level=None, version=None):
     """Valid and generate sources.list for master and containers
 
     :param ubuntu: Ubuntu mirror to use
@@ -1692,7 +1623,7 @@ def _configure_sources_mirror(pkgmgr, ubuntu=None, eole=None, envole=None,
             except RepositoryError as err:
                 msg = _(u'Unable to configure {0} mirror with {1}: {2}')
                 msg = _(u'Not configuring {0} mirror with {1} which seems inaccessible: {2}')
-                log.warn(msg.format(vendor, mirror, normalize(str(err))))
+                log.warning(msg.format(vendor, mirror, normalize(str(err))))
                 continue
         else:
             msg = _(u'Unable to configure APT sources for {0}')
@@ -1762,12 +1693,32 @@ def _configure_sources_mirror(pkgmgr, ubuntu=None, eole=None, envole=None,
         engine.prepare_template(file_info)
         engine.process(file_info, ctx)
 
-    if cdrom and (ubuntu or eole or envole):
-        raise Exception(_('cannot set ubuntu or eole or envole mirror with cdrom parameter'))
+    def _do_template_eolehub(ctx):
+        """Generate container hub specific configuration
+        """
+        if not os.path.isfile(os.path.join(distrib_dir, 'eolehub.common.env')):
+            return True
+
+        file_info = {'name': _EOLEHUB_COMMON_ENV,
+                     'source': os.path.join(templatedir, 'eolehub.common.env'),
+                     'full_name': os.path.join('/', ctx['path'],
+                                               _EOLEHUB_COMMON_ENV[1:]),
+                     'activate': True,
+                     'del_comment': u'',
+                     'mkdir': False,
+                     'rm': False}
+        log.debug(u'Process template for {}'.format(file_info['full_name']))
+        engine.prepare_template(file_info)
+        engine.process(file_info, ctx)
 
     # initialize template engine and context
     creoled_not_available = False
 
+    if version is not None:
+        eole_version = version
+    else:
+        eole_version = EOLE_VERSION
+
     if release is not None:
         eole_release = release
     else:
@@ -1789,98 +1740,95 @@ def _configure_sources_mirror(pkgmgr, ubuntu=None, eole=None, envole=None,
         creoled_not_available = True
         minimal_variables = {'ubuntu_version': UBUNTU_VERSION,
                              'eole_release': eole_release,
-                             'eole_version': EOLE_VERSION}
+                             'eole_version': eole_version}
         engine = CreoleTemplateEngine(force_values=minimal_variables)
         rootctx = {u'name': u'root',
                    u'path': u'/'}
 
-    if not cdrom:
+    if not creoled_not_available:
+        _do_template_aptproxy(rootctx)
+        # reload configuration (FIXME)
+        apt_pkg.config.clear(_APT_PROXY_OPT)
+        apt_pkg.read_config_dir(apt_pkg.config, _APT_CONFIG_DIR)
 
-        if not creoled_not_available:
-            _do_template_aptproxy(rootctx)
-            # reload configuration (FIXME)
-            apt_pkg.config.clear(_APT_PROXY_OPT)
-            apt_pkg.read_config_dir(apt_pkg.config, _APT_CONFIG_DIR)
-
-        # if ubuntu mirror is not forced
-        if ubuntu is None:
-            try:
-                ubuntu = pkgmgr.client.get_creole(u'ubuntu_update_mirrors')
-            except CREOLE_ERRORS as err:
-                if pkgmgr.ignore:
-                    log.warn(_("Unable to find Creole variable for {0}, load default values: {1}").format('Ubuntu', _UBUNTU_MIRRORS))
-                    ubuntu = _UBUNTU_MIRRORS
-                else:
-                    raise err
-        else:
-            ubuntu = [ubuntu]
-
-        # test ubuntu mirror
-        mirror_ubuntu = _test_mirror('Ubuntu', ubuntu)
-
-        # if eole mirror is not forced
-        if eole is None:
-            try:
-                eole = pkgmgr.client.get_creole(u'serveur_maj')
-            except CREOLE_ERRORS as err:
-                if pkgmgr.ignore:
-                    log.warn(_("Unable to find Creole variable for {0}, load default values: {1}").format('EOLE', _EOLE_MIRRORS))
-                    eole = _EOLE_MIRRORS
-                else:
-                    raise err
-        else:
-            eole = [eole]
-        # test eole mirror
-        mirror_eole = _test_mirror('EOLE', eole)
-
-        # Only if envole_version is defined
+    # if ubuntu mirror is not forced
+    if ubuntu is None:
         try:
-            envole_version = pkgmgr.client.get_creole(u'envole_version', None)
+            ubuntu = pkgmgr.client.get_creole(u'ubuntu_update_mirrors')
         except CREOLE_ERRORS as err:
-            envole_version = None
-
-        if envole_version is not None:
-            if envole_release is not None:
-                envole_version = envole_release
-            # if envole mirror is not forced
-            if envole is None:
-                try:
-                    envole = pkgmgr.client.get_creole(u'envole_update_mirrors')
-                except CREOLE_ERRORS as err:
-                    if pkgmgr.ignore:
-                        log.warn(_("Unable to find Creole variable for {0}, load default values: {1}").format('EOLE', _ENVOLE_MIRRORS))
-                        envole = _ENVOLE_MIRRORS
-                    else:
-                        raise err
+            if pkgmgr.ignore:
+                log.warning(_("Unable to find Creole variable for {0}, load default values: {1}").format('Ubuntu', _UBUNTU_MIRRORS))
+                ubuntu = _UBUNTU_MIRRORS
             else:
-                envole = [envole]
-            # test envole mirror
-            mirror_envole = _test_mirror('Envole', envole)
-        else:
-            mirror_envole = None
-
-        mirror_cdrom = None
+                raise err
     else:
-        log.info(_(u'Configuring CDROM'))
-        mirror_ubuntu = None
-        mirror_eole = None
-        mirror_envole = None
+        ubuntu = [ubuntu]
+
+    # test ubuntu mirror
+    mirror_ubuntu = _test_mirror('Ubuntu', ubuntu)
+
+    # if eole mirror is not forced
+    if eole is None:
+        try:
+            eole = pkgmgr.client.get_creole(u'serveur_maj')
+        except CREOLE_ERRORS as err:
+            if pkgmgr.ignore:
+                log.warning(_("Unable to find Creole variable for {0}, load default values: {1}").format('EOLE', _EOLE_MIRRORS))
+                eole = _EOLE_MIRRORS
+            else:
+                raise err
+    else:
+        eole = [eole]
+    # test eole mirror
+    mirror_eole = _test_mirror('EOLE', eole)
+
+    # set container images tag (#34617, #35016)
+    eolehub_tag = {'stable': 'stable',
+                   'proposed': 'testing',
+                   'unstable': 'dev'}['proposed' if mirror_eole.startswith('http://test-eole.ac-dijon.fr') and eole_level == 'stable' else 'unstable' if mirror_eole.startswith('http://test-eole.ac-dijon.fr') and eole_level == 'proposed' else eole_level]
+
+    # Only if envole_version is defined
+    try:
+        envole_version = pkgmgr.client.get_creole(u'envole_version', None)
+    except CREOLE_ERRORS as err:
         envole_version = None
-        mirror_cdrom = 'file:{0}'.format(_MOUNT_POINT)
+
+    if envole_version is not None:
+        if envole_release is not None:
+            envole_version = envole_release
+        # if envole mirror is not forced
+        if envole is None:
+            try:
+                envole = pkgmgr.client.get_creole(u'envole_update_mirrors')
+            except CREOLE_ERRORS as err:
+                if pkgmgr.ignore:
+                    log.warning(_("Unable to find Creole variable for {0}, load default values: {1}").format('EOLE', _ENVOLE_MIRRORS))
+                    envole = _ENVOLE_MIRRORS
+                else:
+                    raise err
+        else:
+            envole = [envole]
+        # test envole mirror
+        mirror_envole = _test_mirror('Envole', envole)
+    else:
+        mirror_envole = None
 
     mirror_variables = {'_eole_level': eole_level,
                         '_envole_level': envole_level,
                         '_eole_release': eole_release,
+                        '_eole_version': eole_version,
                         'envole_version': envole_version,
                         '_mirror_ubuntu': mirror_ubuntu,
                         '_mirror_eole': mirror_eole,
-                        '_mirror_cdrom': mirror_cdrom,
-                        '_mirror_envole': mirror_envole}
+                        '_mirror_envole': mirror_envole,
+                        '_eolehub_tag': eolehub_tag,
+                        }
 
     engine.creole_variables_dict.update(mirror_variables)
 
     try:
         _do_template(rootctx)
+        _do_template_eolehub(rootctx)
         if not creoled_not_available:
             _do_template_additional(rootctx)
             #_do_template_aptproxy(rootctx)
@@ -1894,7 +1842,7 @@ def _configure_sources_mirror(pkgmgr, ubuntu=None, eole=None, envole=None,
                         _do_template_additional(container_infos)
     except Exception as err:
         if pkgmgr.ignore:
-            log.warn(_('cannot build sources.list use current one'))
+            log.warning(_('cannot build sources.list use current one'))
         else:
             raise err
 
@@ -1978,6 +1926,3 @@ def _configure_sources_mirror(pkgmgr, ubuntu=None, eole=None, envole=None,
 
     # delete old cache (#10186)
     pkgmgr.cache = None
-
-    if cdrom:
-        pkgmgr.update()
