@@ -312,13 +312,18 @@ class Command(object):
         :type ctx: `dict`
 
         """
-        name = self._getname(service)
-        msg = u'Restart {0} service {1}{2}'.format(self.module_name,
+        if not isinstance(service, list):
+            name = self._getname(service)
+            msg = u'Restart {0} service {1}{2}'.format(self.module_name,
                                                        name,
                                                        _in_container(ctx))
-        self.message_print(msg, newline = self.fake_restart)
+            self.message_print(msg, newline = self.fake_restart)
+            message = None
+        else:
+            message = u'Restart {0} services{1}'.format(self.module_name, _in_container(ctx))
+            msg = message
         try:
-            self.cmd_restart_service(service, ctx)
+            self.cmd_restart_service(service, ctx, message)
         except ServiceError as err:
             if not self.fake_restart:
                 self.message_print_result(1)
@@ -337,13 +342,18 @@ class Command(object):
         :type ctx: `dict`
 
         """
-        name = self._getname(service)
-        msg = u'Reload {0} service {1}{2}'.format(self.module_name,
+        if not isinstance(service, list):
+            name = self._getname(service)
+            msg = u'Reload {0} service {1}{2}'.format(self.module_name,
                                                       name,
                                                       _in_container(ctx))
-        self.message_print(msg)
+            self.message_print(msg)
+            message = None
+        else:
+            message = u'Reload {0} services{1}'.format(self.module_name, _in_container(ctx))
+            msg = message
         try:
-            self.cmd_reload_service(service, ctx)
+            self.cmd_reload_service(service, ctx, message)
         except ServiceError as err:
             self.message_print_result(1)
             return {'code': 1, 'msg': str(err)}
@@ -367,9 +377,9 @@ class Command(object):
         try:
             cols = self.term_cols
             file_h = open(ti_file, 'w')
-            ctt = 'eole,\n\txenl,\n\thpa=\E[%i%p1%dG,'\
-                  '\n\tsetaf=\E[3%p1%dm,\n\tcols#{0},'\
-                  '\n\top=\E[39;49m,\n'.format(cols)
+            ctt = 'eole,\n\txenl,\n\thpa=\\E[%i%p1%dG,'\
+                  '\n\tsetaf=\\E[3%p1%dm,\n\tcols#{0},'\
+                  '\n\top=\\E[39;49m,\n'.format(cols)
             file_h.write(ctt)
             file_h.close()
             ticcmd = [u'/usr/bin/tic', u'-o', terminfo_dir, ti_file]
