@@ -33,4 +33,6 @@ class Service(ServiceSystemd):
         self.cmd_restart_service(service, ctx, message=message)
 
     def cmd_stop_service(self, service, ctx, message):
-        pass
+        disabled_services = [s for s in service if s['disable']]
+        if disabled_services:
+            super().cmd_stop_service(disabled_services, ctx, message)
